export default class eventSource {
  private url: string;
  private token: string;
  private onMessageCallback: ((data: any) => void) | null = null;
  private onErrorCallback: ((error: any) => void) | null = null;
  private isConnected: boolean = false;
  constructor(url: string, token: string) {
    this.url = url;
    this.token = token;
    this.connect();
  }
  private async connect() {
    try {
      const response = await fetch(this.url, {
        headers: {
          Authorization: `Bearer ${this.token}`,
          Accept: "text/event-stream",
        },
      });
  
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }
  
      const reader = response.body?.getReader();
      if (!reader) throw new Error("No response body");
  
      this.isConnected = true;
  
      let bufferedText = "";
  
      while (this.isConnected) {
        const { done, value } = await reader.read();
        if (done) break;
  
        const text = new TextDecoder().decode(value);
        bufferedText += text;
  
        // Split messages by "\n\n"
        const messages = bufferedText.split("\n\n");
  
        for (let i = 0; i < messages.length - 1; i++) {
          const message = messages[i];
  
          if (message.startsWith("data: ")) {
            const data = message.slice(6); // Remove "data: "
            try {
              // Attempt to parse the data incrementally (streamed JSON)
              const parsedData = JSON.parse(data);
              console.log("Parsed data:", parsedData);
              this.onMessageCallback?.(parsedData);
            } catch (error) {
              console.error("Failed to parse message:", error);
            }
          }
        }
  
        // Keep the last (potentially incomplete) message
        bufferedText = messages[messages.length - 1];
      }
    } catch (error) {
      this.onErrorCallback?.(error);
    }
  }
  
  public onmessage(callback: (data: any) => void) {
    this.onMessageCallback = callback;
  }
  public onerror(callback: (error: any) => void) {
    this.onErrorCallback = callback;
  }
  public close() {
    this.isConnected = false;
  }
}